@@ -1,8 +1,5 @@
 package Dancer::Handler::Debug;
-our $AUTHORITY = 'cpan:SUKRIA';
-#ABSTRACT: a debug handler for easy tracing
-$Dancer::Handler::Debug::VERSION = '1.3514_04'; # TRIAL
-$Dancer::Handler::Debug::VERSION = '1.351404';
+
 use strict;
 use warnings;
 
@@ -16,7 +13,7 @@ use Dancer::SharedData;
 sub run {
     my ($self, $req) = @_;
 
-    my ($method, $path, $query, @env ) = @ARGV;
+    my ($method, $path, $query) = @ARGV;
     my $host    = "127.0.0.1";
     my $port    = "3000";
 
@@ -34,10 +31,9 @@ sub run {
         'SERVER_PROTOCOL' => 'HTTP/1.1',
         'SERVER_SOFTWARE' => 'HTTP::Server::Simple/0.41',
         'SERVER_URL'      => "http://$host:$port/",
-        map { /=/ ? split /=/, $_, 2 : () } @env
     };
 
-    $req = Dancer::Request->new(env => $env);
+    $req = Dancer::Request->new($env);
  
     my $headers = HTTP::Headers->new(
         'User-Agent' => 'curl',
@@ -58,12 +54,12 @@ sub run {
 
 sub start {
     my ($self, $req) = @_;
-    print STDERR ">> Dancer dummy debug server\n" if setting('startup_info');
+    print STDERR ">> Dancer dummy debug server\n" if setting('access_log');
     
     my $dancer = Dancer::Handler::Debug->new();
     my $psgi = sub {
         my $env = shift;
-        my $req = Dancer::Request->new(env => $env);
+        my $req = Dancer::Request->new($env);
         $dancer->handle_request($req);
     };
     $dancer->{psgi_app} = $psgi;
@@ -77,54 +73,26 @@ __END__
 
 =pod
 
-=encoding UTF-8
-
 =head1 NAME
 
 Dancer::Handler::Debug - a debug handler for easy tracing
 
-=head1 VERSION
-
-version 1.3514_04
-
 =head1 DESCRIPTION
 
-When developing a Dancer application, it can be useful to trace precisely what
+When developping a Dancer application, it can be useful to trace precisely what
 happen when a query is processed. This handler is here to provide the developer
 with a way to easily run the dancer application with the Perl debugger.
 
-This handler will process only one query, based on the arguments given on
-the command line, which must follow the pattern
-
-    <verb> <path> <parameter segment> <env variable> <even variable>
+This handler will process ony one query, based on the first argument given on
+the command line ($ARGV[0]).
 
 =head1 USAGE
-
     # in bin/app.pl
     set apphandler => 'Debug';
 
     # then, run the app the following way
-    perl -d bin/app.pl GET '/some/path/to/test' 
-
-    # with arguments
     perl -d bin/app.pl GET '/some/path/to/test' 'with=parameters&other=42'
 
-    # with environment variables
-    perl -d bin/app.pl GET '/some/path/to/test' '' HTTP_X_REQUESTED_WITH=XMLHttpRequest SERVER_NAME=fake
-
 =head1 AUTHORS
 
 Dancer contributors
-
-=head1 AUTHOR
-
-Dancer Core Developers
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2010 by Alexis Sukrieh.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
-=cut
