@@ -1,9 +1,4 @@
-# ABSTRACT: a plugin for adding Ajax route handlers
-
 package Dancer::Plugin::Ajax;
-{
-    $Dancer::Plugin::Ajax::VERSION = '1.9999_02';
-}
 
 use strict;
 use warnings;
@@ -11,38 +6,44 @@ use warnings;
 use Dancer ':syntax';
 use Dancer::Plugin;
 
+register 'ajax' => \&ajax;
 
-hook 'before' => sub {
-    if (request->is_ajax) {
-        content_type('text/xml');
-    }
-};
-
-register 'ajax' => sub {
-    my ($dsl, $pattern, @rest) = @_;
+sub ajax {
+    my ($pattern, @rest) = @_;
 
     my $code;
     for my $e (@rest) { $code = $e if (ref($e) eq 'CODE') }
 
     my $ajax_route = sub {
+        my $layout = setting('layout');
 
         # must be an XMLHttpRequest
-        if (not $dsl->request->is_ajax) {
-            $dsl->pass and return 0;
+        if (not request->is_ajax) {
+            pass and return 0;
         }
 
         # disable layout
-        my $layout = $dsl->setting('layout');
-        $dsl->setting('layout' => undef);
+        setting('layout' => undef);
         my $response = $code->();
-        $dsl->setting('layout' => $layout);
+        setting('layout' => $layout);
         return $response;
     };
 
-    $dsl->any(['get', 'post'] => $pattern, $ajax_route);
-};
+    # rebuild the @rest array with the compiled route handler
+    my @compiled_rest;
+    for my $e (@rest) {
+        if (ref($e) eq 'CODE') {
+            push @compiled_rest, {ajax => 1}, $ajax_route;
+        }
+        else {
+            push @compiled_rest, {ajax => 1}, $e;
+        }
+    }
 
-register_plugin for_versions => [2];
+    any ['get', 'post'] => $pattern, @compiled_rest;
+}
+
+register_plugin;
 1;
 
 __END__
@@ -53,10 +54,6 @@ __END__
 
 Dancer::Plugin::Ajax - a plugin for adding Ajax route handlers
 
-=head1 VERSION
-
-version 1.9999_02
-
 =head1 SYNOPSIS
 
     package MyWebApp;
@@ -79,11 +76,11 @@ The route handler code will be compiled to behave like the following:
 
 =over 4
 
-=item *
+=item * 
 
 Pass if the request header X-Requested-With doesnt equal XMLHttpRequest
 
-=item *
+=item * 
 
 Disable the layout
 
@@ -95,13 +92,6 @@ The action built is a POST request.
 
 =head1 AUTHOR
 
-Dancer Core Developers
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2012 by Alexis Sukrieh.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
+This module has been written by Alexis Sukrieh <sukria@sukria.net>
 
 =cut
